@@ -25,6 +25,7 @@ Single-stream queue-less decodebin
 
 import gobject
 import gst
+from pitivi.stream import get_pad_id, pad_compatible_stream
 
 def is_raw(caps):
     """ returns True if the caps are RAW """
@@ -36,6 +37,14 @@ def is_raw(caps):
     return False
 
 class SingleDecodeBin(gst.Bin):
+    """
+    A variant of decodebin.
+
+    * Only outputs one stream
+    * Doesn't contain any internal queue
+    """
+
+    QUEUE_SIZE = 1 * gst.SECOND
 
     __gsttemplates__ = (
         gst.PadTemplate ("sinkpadtemplate",
@@ -47,11 +56,12 @@ class SingleDecodeBin(gst.Bin):
                          gst.PAD_SOMETIMES,
                          gst.caps_new_any())
         )
-    def __init__(self, caps=None, uri=None, *args, **kwargs):
+    def __init__(self, caps=None, uri=None, stream=None, *args, **kwargs):
         gst.Bin.__init__(self, *args, **kwargs)
         if not caps:
             caps = gst.caps_new_any()
         self.caps = caps
+        self.stream = stream
         self.typefind = gst.element_factory_make("typefind", "internal-typefind")
         self.add(self.typefind)
 
@@ -77,6 +87,8 @@ class SingleDecodeBin(gst.Bin):
 
         self._factories = self._getSortedFactoryList()
 
+        self.debug("stream:%r" % self.stream)
+
 
     ## internal methods
 
@@ -91,7 +103,7 @@ class SingleDecodeBin(gst.Bin):
         Returns the list of demuxers, decoders and parsers available, sorted
         by rank
         """
-        def myfilter(fact):
+        def _myfilter(fact):
             if fact.get_rank() < 64 :
                 return False
             klass = fact.get_klass()
@@ -99,7 +111,7 @@ class SingleDecodeBin(gst.Bin):
                 return False
             return True
         reg = gst.registry_get_default()
-        res = [x for x in reg.get_feature_list(gst.ElementFactory) if myfilter(x)]
+        res = [x for x in reg.get_feature_list(gst.ElementFactory) if _myfilter(x)]
         res.sort(lambda a, b: int(b.get_rank() - a.get_rank()))
         return res
 
@@ -151,6 +163,34 @@ class SingleDecodeBin(gst.Bin):
         for pad in to_connect:
             self._closePadLink(element, pad, pad.get_caps())
 
+    def _isDemuxer(self, element):
+        if not 'Demux' in element.get_factory().get_klass():
+            return False
+
+        potential_src_pads = 0
+        for template in element.get_pad_template_list():
+            if template.direction != gst.PAD_SRC:
+                continue
+
+            if template.presence == gst.PAD_REQUEST or \
+                    "%" in template.name_template:
+                potential_src_pads += 2
+                break
+            else:
+                potential_src_pads += 1
+
+        return potential_src_pads > 1
+
+    def _plugDecodingQueue(self, pad):
+        queue = gst.element_factory_make("queue")
+        queue.props.max_size_time = self.QUEUE_SIZE
+        self.add(queue)
+        queue.sync_state_with_parent()
+        pad.link(queue.get_pad("sink"))
+        pad = queue.get_pad("src")
+
+        return pad
+
     def _tryToLink1(self, source, pad, factories):
         """
         Tries to link one of the factories' element to the given pad.
@@ -160,6 +200,10 @@ class SingleDecodeBin(gst.Bin):
         self.debug("source:%s, pad:%s , factories:%r" % (source.get_name(),
                                                          pad.get_name(),
                                                          factories))
+
+        if self._isDemuxer(source):
+            pad = self._plugDecodingQueue(pad)
+
         result = None
         for factory in factories:
             element = factory.create()
@@ -172,6 +216,7 @@ class SingleDecodeBin(gst.Bin):
                 continue
 
             self.add(element)
+            element.set_state(gst.STATE_READY)
             try:
                 pad.link(sinkpad)
             except:
@@ -202,10 +247,17 @@ class SingleDecodeBin(gst.Bin):
         if caps.is_any():
             self.log("type is not know yet, waiting")
             return
-        if caps.intersect(self.caps):
+
+        self.debug("stream %r" % (self.stream))
+        if caps.intersect(self.caps) and (self.stream is None or
+                (self.stream.pad_name == get_pad_id(pad))):
             # This is the desired caps
             if not self._srcpad:
                 self._wrapUp(element, pad)
+        elif is_raw(caps) and pad_compatible_stream(pad, self.stream):
+            self.log ("not the target stream, but compatible")
+            if not self._srcpad:
+                self._wrapUp(element, pad)
         elif is_raw(caps):
             self.log("We hit a raw caps which isn't the wanted one")
             # FIXME : recursively remove everything until demux/typefind
@@ -230,8 +282,8 @@ class SingleDecodeBin(gst.Bin):
         if self._srcpad:
             return
         self._markValidElements(element)
-        self._removeUnusedElements(self.typefind)
-        self.log("ghosting pad %s" % pad.get_name)
+        gobject.idle_add(self._removeUnusedElements, self.typefind)
+        self.log("ghosting pad %s" % pad.get_name())
         self._srcpad = gst.GhostPad("src", pad)
         self._srcpad.set_active(True)
         self.add_pad(self._srcpad)
@@ -254,16 +306,17 @@ class SingleDecodeBin(gst.Bin):
         """
         Remove unused elements connected to srcpad(s) of element
         """
-        self.log("element:%s" % element)
+        self.log("element:%r" % element)
         for pad in element.src_pads():
             if pad.is_linked():
                 peer = pad.get_peer().get_parent()
-                self._removeUnusedElements(peer)
-                if not peer in self._validelements:
-                    self.log("removing %s" % peer.get_name())
-                    pad.unlink(pad.get_peer())
-                    peer.set_state(gst.STATE_NULL)
-                    self.remove(peer)
+                if isinstance(peer, gst.Element):
+                    self._removeUnusedElements(peer)
+                    if not peer in self._validelements:
+                        self.log("removing %s" % peer.get_name())
+                        pad.unlink(pad.get_peer())
+                        peer.set_state(gst.STATE_NULL)
+                        self.remove(peer)
 
     def _cleanUp(self):
         self.log("")
@@ -280,7 +333,7 @@ class SingleDecodeBin(gst.Bin):
     def do_change_state(self, transition):
         self.debug("transition:%r" % transition)
         res = gst.Bin.do_change_state(self, transition)
-        if transition in [gst.STATE_CHANGE_PAUSED_TO_READY, gst.STATE_CHANGE_READY_TO_NULL]:
+        if transition == gst.STATE_CHANGE_PAUSED_TO_READY:
             self._cleanUp()
         return res
 
